Module: Rjq::Opcodes

Defined in:
lib/rjq/opcodes.rb

Constant Summary collapse

ALL =
%i[
  load_input load_const string_interp format variable field index_const index_filter slice_const slice_filter
  each path optional pipe append binding array object branch try reduce foreach label break unary binary assign
  call tail_call recurse scoped_def
].freeze
MNEMONICS =
ALL.to_h { |opcode| [opcode, opcode.to_s] }.freeze